

import os

import pytest
from .httpserver import HTTPServer


class Plugin:
    SERVER = None


class PluginHTTPServer(HTTPServer):
    def start(self):
        super().start()
        Plugin.SERVER = self

    def stop(self):
        super().stop()
        Plugin.SERVER = None


def get_httpserver_listen_address():
    listen_host = os.environ.get("PYTEST_HTTPSERVER_HOST")
    listen_port = os.environ.get("PYTEST_HTTPSERVER_PORT")
    if listen_port:
        listen_port = int(listen_port)

    return (listen_host, listen_port)


@pytest.fixture
def httpserver_listen_address():
    return get_httpserver_listen_address()


@pytest.fixture
def httpserver(httpserver_listen_address):
    if Plugin.SERVER:
        Plugin.SERVER.clear()
        yield Plugin.SERVER
        return

    host, port = httpserver_listen_address
    if not host:
        host = HTTPServer.DEFAULT_LISTEN_HOST
    if not port:
        port = HTTPServer.DEFAULT_LISTEN_PORT

    server = PluginHTTPServer(host=host, port=port)
    server.start()
    yield server


def pytest_sessionfinish(session, exitstatus):  # pylint: disable=unused-argument
    if Plugin.SERVER is not None:
        Plugin.SERVER.clear()
        if Plugin.SERVER.is_running():
            Plugin.SERVER.stop()



# Run this code as 'pytest example_pytest.py'

import pytest
import requests
from pytest_httpserver import HTTPServer

# specify where the server should bind to
# you can return 0 as the port, in this case it will bind to a free (ephemeral) TCP port
@pytest.fixture
def httpserver_listen_address():
    return ("127.0.0.1", 8000)


# specify httpserver fixture
def test_oneshot_and_permanent_happy_path1(httpserver: HTTPServer):
    # define some request handlers
    # more details in the documentation
    httpserver.expect_request("/permanent").respond_with_data("OK permanent")
    httpserver.expect_oneshot_request("/oneshot1").respond_with_data("OK oneshot1")
    httpserver.expect_oneshot_request("/oneshot2").respond_with_data("OK oneshot2")

    # query those handlers with a real HTTP client (requests in this example but could by anything)
    # the 'url_for' method  formats the final URL, so there's no need to wire-in any ports
    assert requests.get(httpserver.url_for("/permanent")).text == "OK permanent"
    assert requests.get(httpserver.url_for("/oneshot1")).text == "OK oneshot1"
    assert requests.get(httpserver.url_for("/oneshot2")).text == "OK oneshot2"
    assert requests.get(httpserver.url_for("/permanent")).text == "OK permanent"

    assert len(httpserver.oneshot_handlers) == 0



import requests
from pytest_httpserver import HTTPServer
from werkzeug.http import parse_dict_header

from pytest_httpserver.httpserver import HeaderValueMatcher


def test_custom_headers(httpserver: HTTPServer):
    headers_with_values_in_direct_order = {'Custom': 'Scheme key0="value0", key1="value1"'}
    httpserver.expect_request(uri='/', headers=headers_with_values_in_direct_order).respond_with_data('OK')
    response = requests.get(httpserver.url_for('/'), headers=headers_with_values_in_direct_order)
    assert response.status_code == 200
    assert response.text == 'OK'

    # By default different order of items in header value dicts means different header values
    headers_with_values_in_modified_order = {'Custom': 'Scheme key1="value1", key0="value0"'}
    response = requests.get(httpserver.url_for('/'), headers=headers_with_values_in_modified_order)
    assert response.status_code == 500

    # Define header_value_matcher that ignores the order of items in header value dict
    def custom_header_value_matcher(actual: str, expected: str) -> bool:
        actual_scheme, _, actual_dict_str = actual.partition(' ')
        expected_scheme, _, expected_dict_str = expected.partition(' ')
        actual_dict = parse_dict_header(actual_dict_str)
        expected_dict = parse_dict_header(expected_dict_str)
        return actual_scheme == expected_scheme and actual_dict == expected_dict

    matchers = HeaderValueMatcher.DEFAULT_MATCHERS.copy()
    matchers['Custom'] = custom_header_value_matcher
    header_value_matcher = HeaderValueMatcher(matchers)

    httpserver.handlers.clear()
    httpserver.expect_request(
        uri='/',
        headers=headers_with_values_in_direct_order,
        header_value_matcher=header_value_matcher
    ).respond_with_data('OK')
    response = requests.get(httpserver.url_for('/'), headers=headers_with_values_in_modified_order)
    assert response.status_code == 200
    assert response.text == 'OK'


# See https://en.wikipedia.org/wiki/Digest_access_authentication
def test_authorization_headers(httpserver: HTTPServer):
    headers_with_values_in_direct_order = {
        'Authorization': ('Digest username="Mufasa",'
                          'realm="testrealm@host.com",'
                          'nonce="dcd98b7102dd2f0e8b11d0f600bfb0c093",'
                          'uri="/dir/index.html",'
                          'qop=auth,'
                          'nc=00000001,'
                          'cnonce="0a4f113b",'
                          'response="6629fae49393a05397450978507c4ef1",'
                          'opaque="5ccc069c403ebaf9f0171e9517f40e41"')
    }
    httpserver.expect_request(uri='/', headers=headers_with_values_in_direct_order).respond_with_data('OK')
    response = requests.get(httpserver.url_for('/'), headers=headers_with_values_in_direct_order)
    assert response.status_code == 200
    assert response.text == 'OK'

    headers_with_values_in_modified_order = {
        'Authorization': ('Digest qop=auth,'
                          'username="Mufasa",'
                          'nonce="dcd98b7102dd2f0e8b11d0f600bfb0c093",'
                          'uri="/dir/index.html",'
                          'nc=00000001,'
                          'realm="testrealm@host.com",'
                          'response="6629fae49393a05397450978507c4ef1",'
                          'cnonce="0a4f113b",'
                          'opaque="5ccc069c403ebaf9f0171e9517f40e41"')
    }
    response = requests.get(httpserver.url_for('/'), headers=headers_with_values_in_modified_order)
    assert response.status_code == 200
    assert response.text == 'OK'



"""
This is package provides the main API for the pytest_httpserver package.

"""

# flake8: noqa

from .httpserver import HTTPServer
from .httpserver import HTTPServerError, Error, NoHandlerError
from .httpserver import WaitingSettings, HeaderValueMatcher, RequestHandler
from .httpserver import URIPattern, URI_DEFAULT, METHOD_ALL



#!.venv/bin/python3

import urllib.request
import urllib.error

from pytest_httpserver import HTTPServer


server = HTTPServer(port=4000)
server.expect_request("/foobar").respond_with_json({"foo": "bar"})
server.start()
try:
    print(urllib.request.urlopen("http://localhost:4000/foobar?name=John%20Smith&age=123").read())
except urllib.error.HTTPError as err:
    print(err)

server.stop()




import ssl
import os
from os.path import join as pjoin

import requests
from pytest_httpserver import HTTPServer

test_dir = os.path.dirname(os.path.realpath(__file__))
assets_dir = pjoin(test_dir, "assets")


def test_ssl():
    protocol = None
    for name in ("PROTOCOL_TLS_SERVER", "PROTOCOL_TLS", "PROTOCOL_TLSv1_2"):
        if hasattr(ssl, name):
            protocol = getattr(ssl, name)
            break

    assert protocol is not None, "Unable to obtain TLS protocol"

    context = ssl.SSLContext(protocol)

    server_crt = pjoin(assets_dir, "server.crt")
    server_key = pjoin(assets_dir, "server.key")
    root_ca = pjoin(assets_dir, "rootCA.crt")
    context.load_cert_chain(server_crt, server_key)

    with HTTPServer(ssl_context=context, port=4433) as httpserver:
        httpserver.expect_request("/foobar").respond_with_json({"foo": "bar"})
        assert httpserver.is_running()
        assert requests.get(httpserver.url_for("/foobar"), verify=root_ca).json() == {'foo': 'bar'}



#!/usr/bin/env python3

from setuptools import setup, find_packages


DESCRIPTION = open("README.md").read()

setup(
    name="pytest_httpserver",
    version="0.3.7",
    url="https://www.github.com/csernazs/pytest-httpserver",
    packages=find_packages(),
    author="Zsolt Cserna",
    author_email="cserna.zsolt@gmail.com",
    description="pytest-httpserver is a httpserver for pytest",
    entry_points={"pytest11": ["pytest_httpserver = pytest_httpserver.pytest_plugin"]},
    long_description=DESCRIPTION,
    long_description_content_type="text/markdown",
    python_requires=">=3.4",
    install_requires=[
        "typing;python_version<'3.5'",
        "werkzeug"
    ],
    extras_require={
        "dev": [
            "flake8",
            "wheel",
            "pytest",
            "pytest-cov",
            "coverage",
            "ipdb",
            "requests",
            "sphinx",
            "sphinx_rtd_theme",
            "reno",
            "autopep8",
        ],
        "test": [
            "pytest",
            "pytest-cov",
            "coverage",
            "requests",
        ],
    },
    tests_require=["pytest", "requests"],
    license="MIT",
    platforms="any",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Framework :: Pytest",
    ],
)
